from .category import Category
from .employment import EmploymentType
from .jobs import GovJobDetails, Identifier, Job, JobDetails, JobType, Location

__all__ = [
    "Category",
    "EmploymentType",
    "GovJobDetails",
    "Identifier",
    "Job",
    "JobDetails",
    "JobType",
    "Location",
]



import json
import re

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Identifier, JobDetails, Location
from pyethiojobs.utils import remove_newline


class GetJobsDetails(Scaffold):
    def _get_job_details(self, html) -> JobDetails:
        soup = self.soup(html)
        job = json.loads(
            re.sub(
                r";\s*\"(.+)\"\.\s",
                r"“\1“:",
                remove_newline(
                    soup.find("script", {"type": "application/ld+json"}).text
                ),
            )
        )
        views = soup.find("span", {"class": "jobs_by"}).text
        job_id = int(re.search(r"(\d+)\s+\|", views).group(1))
        try:
            views = int(re.search(r"(\d+)\s+Views", views).group(1))
        except AttributeError:
            views = 0
        div = soup.find("div", {"id": "col-wide"})
        salary = div.find("div", string="Salary:")
        salary = salary.find_next_sibling().text if salary else None
        category = div.find("div", string="Category:")
        category = category.find_next_sibling().text if category else None
        location = div.find("div", string="Location:")
        location = (
            location.find_next_sibling().get_text(strip=True) if location else None
        )
        career_level = div.find("div", string="Career Level:")
        career_level = career_level.find_next_sibling().text if career_level else None
        employment_type = div.find("div", string="Employment Type:")
        employment_type = (
            employment_type.find_next_sibling().text if employment_type else None
        )
        return JobDetails(
            id=job_id,
            views=views,
            title=job["title"],
            work_place=location,
            location=Location(
                address_locality=job["jobLocation"]["address"][
                    "addressLocality"
                ].strip(),
                address_region=job["jobLocation"]["address"]["addressRegion"].strip(),
                address_country=job["jobLocation"]["address"]["addressCountry"].strip(),
            ),
            date_posted=job["datePosted"],
            valid_through=job["validThrough"],
            type=employment_type,
            hiring_organization=job["hiringOrganization"]["name"],
            link=self._BASE_URL.format(f"display-job/{job_id}"),
            description=job["description"],
            identifier=Identifier(
                name=job["identifier"]["name"],
                id=job["identifier"]["value"],
            ),
            print_link=self._BASE_URL.format(f"print-job/?listing_id={job_id}"),
            category=category,
            salary=salary,
            career_level=career_level,
        )



from typing import Union

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import JobDetails


class ConvertToPdf(Scaffold):
    async def convert_to_pdf(self, job: Union[str, JobDetails], filename: str) -> str:
        # link = job.print_link if isinstance(job, JobDetails) else job
        # html = HTML(link)
        # html.write_pdf(filename)
        # return filename
        pass



from pyethiojobs import types


class JobDetails:
    def __init__(
        self,
        id: int,
        title: str,
        description: str,
        link: str,
        views: int,
        category: str,
        salary: str,
        print_link: str,
        identifier: "types.Identifier",
        date_posted: str,
        valid_through: str,
        hiring_organization: str,
        location: "types.Location",
        work_place: str,
        type: str,
        career_level: str,
    ):
        self.id = id
        self.print_link = print_link
        self.salary = salary
        self.category = category
        self.views = views
        self.work_place = work_place
        self.title = title
        self.description = description
        self.link = link
        self.identifier = identifier
        self.date_posted = date_posted
        self.valid_through = valid_through
        self.hiring_organization = hiring_organization
        self.location = location
        self.type = type
        self.career_level = career_level

    def dict(self):
        return {
            "id": self.id,
            "title": self.title,
            "description": self.description,
            "link": self.link,
            "views": self.views,
            "category": self.category,
            "salary": self.salary,
            "print_link": self.print_link,
            "identifier": self.identifier.__dict__,
            "date_posted": self.date_posted,
            "valid_through": self.valid_through,
            "hiring_organization": self.hiring_organization,
            "location": self.location.__dict__,
            "work_place": self.work_place,
            "type": self.type,
            "career_level": self.career_level,
        }



from .category import Categories
from .decorators import Decorators
from .gov import Gov
from .job import Jobs
from .search import Search
from .utlis import Utils


class Methods(Search, Categories, Utils, Gov, Decorators, Jobs):
    pass



from bs4 import Tag

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import GovJobDetails, JobType


class GetGovJobsDetails(Scaffold):
    def _get_gov_job_details(self, html) -> GovJobDetails:
        soup = self.soup(html)
        job_title = soup.find("h1", {"class": "details-header__title "}).text
        location, gov_type = soup.find_all(
            "li",
            {"class": "listing-item__info--item listing-item__info--item-location"},
        )
        company = soup.find(
            "li", {"class": "listing-item__info--item listing-item__info--item-company"}
        )
        deadline = soup.find(
            "li", {"class": "listing-item__info--item listing-item__info--item-date"}
        )
        job_: Tag = soup.find("div", {"class": "job-type"})
        job_t = job_.find_all("span", {"class": "job-type__value"})
        salary, working_time = job_t[:2]
        type = ", ".join([j.text.strip() for j in job_t[2:]])
        # Todo separate the discription to Education level, Work experience, Instruction ...
        description = soup.find("div", {"class": "pull-left details-body__left"})
        job = JobType(salary=salary.text, working_time=working_time.text, type=type)
        return GovJobDetails(
            job_title=job_title.strip(),
            company=company.text.strip(),
            location=location.text.strip(),
            gov_type=gov_type.text.strip(),
            deadline=deadline.text.strip(),
            job_type=job,
            description=description.text.strip(),
        )



from typing import Callable

from pyethiojobs.filters import Filter
from pyethiojobs.scaffold import Scaffold


class OnNewJob(Scaffold):
    def on_new_jobs(self, filters: Filter = None) -> Callable:
        """Decorator for handling when a new job is available
        Example:
            .. code-block:: python
                :emphasize-lines: 4-5
                ...
                app = pyEthioJobs(...)
                ...
                @app.on_new_jobs()
                async def handler(inst: pyEthioJobs , job: Job):
                    print(message.__dict__)
                ...
                app.run(...)
        """

        method = "NEW_JOB_HANDLER"

        def decorator(func: Callable) -> Callable:
            if self is not None:
                self._on_event_update.set_handler(method, func, filters)
            return func

        return decorator



import httpx
from bs4 import BeautifulSoup

from .errors import UnknownError
from .handler import HandlersHolder
from .methods import Methods
from .scaffold import Scaffold


class EthioJobs(Methods, Scaffold):
    def __init__(self):
        super().__init__()
        self._session = httpx.AsyncClient()
        self._soup = BeautifulSoup
        self._on_event_update = HandlersHolder(self)

    def soup(self, html: str) -> BeautifulSoup:
        return self._soup(html, "lxml")

    async def _process_request(
        self, url: str, method="GET", **kwargs
    ) -> httpx.Response:
        if "http" in url:
            return await self._session.request(method, url, timeout=30, **kwargs)
        try:
            return await self._session.request(
                method=method,
                url=self._BASE_URL.format(url),
                timeout=30,
                **kwargs,
            )
        except httpx.HTTPError:
            return await self._session.request(
                method=method,
                url=self._BASE_URL.format(url),
                timeout=30,
                **kwargs,
            )
        except Exception as e:
            raise UnknownError(str(e))



class Location:
    def __init__(
        self,
        address_locality: str,
        address_region: str,
        address_country: str,
    ):
        self.address_locality = address_locality
        self.address_region = address_region
        self.address_country = address_country



from .convert_to_pdf import ConvertToPdf
from .get_job_detail import GetJobsDetails
from .get_jobs import GetJobs
from .run import Run


class Utils(GetJobs, GetJobsDetails, Run, ConvertToPdf):
    pass



from typing import List

from bs4 import Tag

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types.jobs import GovJob


class GetGovernmentJobs(Scaffold):
    async def get_gov_jobs(self) -> List[GovJob]:
        res = await self._process_request(url="")
        soup = self.soup(res.text)
        div: Tag = soup.find("div", {"id": "public_latest"})
        gov_jobs = []
        for job in div.find_all("div", {"class": "single_listing col-md-6"}):
            anchor = job.find("a")
            link = anchor.get("href")
            job_ = anchor.text
            company = job.find("p", {"class": "no-marign"})
            posted = job.find("span", {"class": "pull-right"})
            gov_jobs.append(
                GovJob(
                    company=company.text.strip(),
                    link=link,
                    job=job_.strip(),
                    posted=posted,
                    base=self,
                )
            )
        return gov_jobs



from setuptools import setup, find_packages


def read_file(filename: str) -> str:
    with open(filename, encoding="utf-8", mode="r") as f:
        return f.read()


setup(
    name="pyEthioJobs",
    version="0.0.7",
    packages=find_packages(),
    url="https://github.com/wizkiye/pyEthioJobs",
    license="",
    author="Kidus",
    author_email="wizkiye@gmail.com",
    long_description=read_file("README.md"),
    install_requires=[
        "bs4>=0.0.1",
        "asyncio>=3.4.3",
        "httpx>=0.24.1",
        "lxml>=4.9.3",
    ],
    long_description_content_type="text/markdown",
    description="A python package to scrape jobs from ethiojobs.net",
)



from typing import List

from bs4 import Tag

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Category


class GetCategories(Scaffold):
    async def get_categories(self) -> List[Category]:
        """
        Get all categories from the site.
        """
        res = await self._process_request(url="")
        soup = self.soup(res.text)
        div: Tag = soup.find("div", {"id": "job_category"})
        uls: Tag = div.find("ul", {"class": "list-group"})
        categories = []
        for anchor in uls.find_all("a", {"class": "list-group-item"}):
            name = anchor["title"]
            link = anchor["href"]
            count = int(anchor.find("span").text)
            categories.append(Category(name=name, link=link, count=count))
        return categories



import json
import re
from typing import List

from bs4 import Tag

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Identifier, Job, Location
from pyethiojobs.utils import remove_newline


class GetJobs(Scaffold):
    def _get_jobs(self, html) -> List[Job]:
        if "There are no postings meeting the criteria you specified" in html:
            return []
        soup = self.soup(
            html.split("<!-- start: JOB INFO -->")[1].split("<!-- end: JOB INFO -->")[0]
        )
        tbody: Tag = soup.find("tbody", {"class": "searchResultsJobs"})
        json_jobs = tbody.find_all("script", {"type": "application/ld+json"})
        trs = tbody.find_all("tr", class_="evenrow")
        jobs = []
        for job, tr in zip(json_jobs, trs):
            job = json.loads(remove_newline(job.text))
            link = tr.find("a", {"title": "View Job"})
            work_place = tr.find("span", {"class": "work-palce captions-field"})
            experience = tr.find(
                "span",
                {"class": "captions-field"},
                text=re.compile("Level"),
                recursive=True,
            )
            experience = experience.text.strip() if experience else None
            jobs.append(
                Job(
                    job_id=re.search(r"\/(\d+)\/", link["href"]).group(1),
                    title=job["title"].strip(),
                    description=job["description"],
                    identifier=Identifier(
                        name=job["identifier"]["name"],
                        id=job["identifier"]["value"],
                    ),
                    experience=experience,
                    location=Location(
                        address_locality=job["jobLocation"]["address"][
                            "addressLocality"
                        ].strip(),
                        address_region=job["jobLocation"]["address"][
                            "addressRegion"
                        ].strip(),
                        address_country=job["jobLocation"]["address"][
                            "addressCountry"
                        ].strip(),
                    ),
                    date_posted=job["datePosted"],
                    valid_through=job["validThrough"],
                    type=job.get("employmentType"),
                    hiring_organization=job["hiringOrganization"]["name"],
                    link=link["href"],
                    base=self,
                    work_place=work_place.text.strip(),
                )
            )
        return jobs



from typing import List

from pyethiojobs import types


class Category:
    def __init__(
        self,
        name: str,
        link: str,
        count: int,
        base=None,
    ):
        self.name = name
        self.link = link
        self.count = count
        self._base = base

    async def get_jobs(self) -> List["types.Job"]:
        pass
